C$Procedure  XDDA  ( list voxels intersected by a ray )
 
      SUBROUTINE XDDA ( OBS, RAY, NVOX, NVX, VXLST2 )
 
C$ Abstract
C
C     Given a ray and a voxel grid, return a list of voxels that
C     it intersects.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE VOXEL
C
C$ Declarations
 
      IMPLICIT              NONE
 
      INTEGER               NVX
      INTEGER               NVOX     ( 3 )
      INTEGER               VXLST2   ( * )
 
      DOUBLE PRECISION      OBS      ( 3 )
      DOUBLE PRECISION      RAY      ( 3 )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     OBS        I   Voxel grid coordinates of ray's starting point.
C     RAY        I   Direction vector of ray.
C     NVOX       O   Dimensions of grid in voxel units.
C     NVX        O   Number of voxels in the VXLST2 list.
C     VXLST2     O   List of voxels intersected by ray.
C
C$ Detailed_Input
C
C     OBS        Voxel grid coordinates of ray's starting point.
C
C     RAY        Direction vector of ray.
C
C$ Detailed_Output
C
C     NVOX       Dimensions of grid in voxel units.
C
C     NVX        Number of voxels in the VXLST2 list.
C
C     VXLST2     List of voxels intersected by ray.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1)
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 03-FEB-1999 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     list voxels intersected by a ray
C
C-&
 
      INTEGER               I
      INTEGER               IX
      INTEGER               IY
      INTEGER               IZ
      INTEGER               INDXYZ
      INTEGER               STEP     ( 3 )
      INTEGER               VXLST    ( 3, 10000 )
 
      LOGICAL               INGRD
      LOGICAL               VZERO
      LOGICAL               RETURN
 
      DOUBLE PRECISION      X0
      DOUBLE PRECISION      Y0
      DOUBLE PRECISION      Z0
      DOUBLE PRECISION      SXY
      DOUBLE PRECISION      SXZ
      DOUBLE PRECISION      YERR
      DOUBLE PRECISION      ZERR
 
C
C     Standard SPICELIB error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'XDDA' )
 
C
C     Check if ray direction vector is a zero vector.
C
 
      IF ( VZERO( RAY ) ) THEN
         WRITE(*,*) 'xddf: ray is the zero vector'
         STOP
      END IF
 
C
C     Which voxel does the point start in?
C
 
      IX = 1 + INT( OBS(1) )
      IY = 1 + INT( OBS(2) )
      IZ = 1 + INT( OBS(3) )
 
      CALL IPACK3 ( IX, IY, IZ, VXLST(1,1) )
 
      IF ( .NOT. INGRD(NVOX, VXLST(1,1)) ) THEN
         WRITE(*,*) 'XDDA: starting point not in grid.'
         WRITE(*,*) 'ix, iy, iz = ', IX, IY, IZ
         WRITE(*,*) 'nvox = ', NVOX
         WRITE(*,*) 'vxlst = ', (VXLST(I,1),I=1,3)
         WRITE(*,*) 'obs = ', OBS
         STOP
      END IF
 
      NVX = 1
 
C
C     Relative location of starting point within the voxel.
C
 
      X0 = DMOD( OBS(1), 1.D0 )
      Y0 = DMOD( OBS(2), 1.D0 )
      Z0 = DMOD( OBS(3), 1.D0 )
 
C
C     Determine the error term initial values and increments.
C
 
C
C     Different quadrant directions result in slightly
C     different initial conditions; the only effect of
C     quadrant in the main loop is in the step direction.
C
 
      YERR = 9.D9
      ZERR = 9.D9
 
      IF ( RAY(2) .NE. 0.D0 ) THEN
         SXY = DABS(RAY(1)/RAY(2))
         IF ( RAY(1) .GT. 0.D0 ) THEN
            IF( RAY(2) .GT. 0.D0 ) THEN
               YERR = SXY*(1.D0 - Y0) + X0 - 1.D0
            ELSE
               YERR = SXY*Y0 + X0 - 1.D0
            END IF
         ELSE
            IF( RAY(2) .GT. 0.D0 ) THEN
               YERR = SXY*(1.D0 - Y0) - X0
            ELSE
               YERR = SXY*Y0 - X0
            END IF
         END IF
      ELSE
         SXY = 0.D0
      END IF
 
 
      IF ( RAY(3) .NE. 0.D0 ) THEN
         SXZ = DABS(RAY(1)/RAY(3))
         IF ( RAY(1) .GT. 0.D0 ) THEN
            IF( RAY(3) .GT. 0.D0 ) THEN
               ZERR = SXZ*(1.D0 - Z0) + X0 - 1.D0
            ELSE
               ZERR = SXZ*Z0 + X0 - 1.D0
            END IF
         ELSE
            IF( RAY(3) .GT. 0.D0 ) THEN
               ZERR = SXZ*(1.D0 - Z0) - X0
            ELSE
               ZERR = SXZ*Z0 - X0
            END IF
         END IF
      ELSE
         SXZ = 0.D0
      END IF
 
C
C     Set the step increment's sign for each coordinate.
C
 
      DO I = 1, 3
         IF ( RAY(I) .GT. 0.D0 ) THEN
            STEP(I) =  1
         ELSE IF ( RAY(I) .LT. 0.D0 ) THEN
            STEP(I) = -1
         ELSE IF ( RAY(I) .EQ. 0.D0 ) THEN
            STEP(I) =  0
         END IF
      END DO
 
C
C     Follow the ray until the voxel grid is exited.
C
 
      DO WHILE ( INGRD( NVOX, VXLST(1,NVX)) )
 
 
         IF ( (YERR .LT. 0.D0) .OR. (ZERR .LT. 0.D0) ) THEN
 
C
C     Ray has crossed over into the next voxel in Y or Z.
C
 
            IF ( YERR .LT. ZERR ) THEN
 
               IY   = IY + STEP(2)
               YERR = YERR + SXY
               NVX  = NVX + 1
 
               CALL IPACK3 ( IX, IY, IZ, VXLST(1,NVX) )
 
            ELSE
 
               IZ   = IZ + STEP(3)
               ZERR = ZERR + SXZ
               NVX = NVX + 1
 
               CALL IPACK3 ( IX, IY, IZ, VXLST(1,NVX) )
 
            END IF
 
         ELSE
 
C
C     No change in Y or Z, step in X.
C
 
            IX = IX + STEP(1)
            NVX = NVX + 1
            CALL IPACK3 ( IX, IY, IZ, VXLST(1,NVX) )
            YERR = YERR - 1.D0
            ZERR = ZERR - 1.D0
 
         END IF
 
      END DO
 
C
C     Subtract off one voxel (it's outside the grid).
C
 
      NVX = NVX - 1
 
      DO I = 1, NVX
         VXLST2(I) = INDXYZ ( VXLST(1,I), NVOX )
      END DO
 
C
C     Standard SPICE error handling.
C
 
      CALL CHKOUT ( 'XDDA' )
 
 
      RETURN
      END
